#
#  FILE NAME:
#      exception.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# P3 imports
from p3.base.string_helpers import uni_str


class GenericException( Exception ):
    def __init__( self, module_name=None, error_code=None, error_message=None ):
        self.module_name   = self._sanitize_str( module_name )
        self.error_code    = self._sanitize_int( error_code )
        self.error_message = self._sanitize_str( error_message )


    #* If the attribute is narrow or wide string, convert it to narrow string.
    #  Otherwise return None.
    @classmethod
    def _sanitize_str( cls, s ):
        if isinstance( s, basestring ):
            return uni_str( s )
        else:
            return None


    @classmethod
    def _sanitize_int( cls, i ):
        if isinstance( i, int ):
            return i
        else:
            return None


    def __str__( self ):
        return ("%s#%s%s" % (
                ('?' if self.module_name is None else self.module_name),
                ('?' if self.error_code is None else str(self.error_code)),
                ('' if self.error_message is None else ': %s' % self.error_message)))


class BaseModuleError( GenericException ):
    class codes(object):
        enum_definition_error  = 1
        enum_const_error       = 2
        invalid_directory_path = 3
        invalid_import_path    = 4
        required_module_not_available = 5

    def __init__( self, code=None, msg=None ):
        super( BaseModuleError, self ).__init__( "base", code, msg )
