#
#  FILE NAME:
#      handler.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

from p3.base import challenge_module

# Standard library imports:
import Cookie
import logging

# Third party imports:
challenge_module('django')
import django.http
import django.template.loader

# P3 imports:
from p3.base import import_object, Enum
from p3.logic.response import PageResponse, RedirectResponse


_LOG = logging.getLogger( "p3.web" )


## Convert cookies stored in logic_response object (p3.logic.response.HttpResponse like) and add it
#   to the django_response object.
def django_transfer_cookies( django_response, logic_response ):
    ## @note morsel is expected to be Cookie.Morsel like object.
    for morsel in logic_response.cookies:
        django_response.set_cookie(
            morsel.key, morsel.value, morsel['max-age'], morsel['expires'], morsel['path'],
            morsel['domain'], morsel['secure'] )


def _render_django_default( response ):
    return django.template.loader.render_to_string(
        response.template_name, response.template_context )


def serve_django_page( request, logic, render_cb=_render_django_default ):
    _LOG.info( "Serving page generated by '%s' class", logic )
    
    cls = import_object( logic )
    obj = cls( request )
    obj.execute()

    if isinstance( obj.response, PageResponse ):
        rendered = render_cb( obj.response )
        response = django.http.HttpResponse( rendered, 200, content_type='text/html' )
    elif isinstance( obj.response, RedirectResponse ):
        response = django.http.HttpResponseRedirect( obj.response.url )
    else:
        assert False, "Unsupported response type."

    django_transfer_cookies( response, obj.response )

    return response
